{ ECHMRCI3_Intf

  Pascal implementation of ECHMRCI3.DLL, the remote control interface DLL
  for Help & Manual 3.

   2002 EC Software. All rights reserved.

  This product and it's source code is protected by copyright patents, laws and
  international copyright treaties, as well as other intellectual property
  laws and treaties. The product is licensed, not sold.

  The source code and sample programs in this package or parts hereof
  as well as the documentation shall not be copied, modified or redistributed
  without permission, explicit or implied, of the author.


  EMail: info@ec-software.com
  Internet: http://www.ec-software.com


  Disclaimer of Warranty
  ----------------------

  THIS SOFTWARE AND THE ACCOMPANYING FILES ARE PROVIDED "AS IS" AND
  WITHOUT WARRANTIES OF ANY KIND WHETHER EXPRESSED OR IMPLIED.

  In no event shall the author be held liable for any damages whatsoever,
  including without limitation, damages for loss of business profits,
  business interruption, loss of business information, or any other loss
  arising from the use or inability to use the software. }


unit ECHMRCI3_Intf;

interface

uses
  Windows;

{$I ECHMRCI3_DEF.INC}

const
  DLLName = 'ECHMRCI3.DLL';

type
  { THMInstance is the communication ID of a "server", a running instance of Help & Manual. }

  THMInstance = Longint;

  { THMRCIDataCallback:
    Callback function for various enumerations, used by ListServers, GetTopicList
    and other procedures. The callback function must return TRUE in order to continue.
    AppData specifies an application defined callback value. This parameter is
    typically used to specify a pointer to a list the values will be added to. }

  THMRCIDataCallback = function(pBuffer: pointer; iData: longint; AppData: longint): bool; cdecl;

  { THMRCIEventCallback:
    Callback function for asynchronous events. hInstance is the server ID. The following
    events are currently implemented:

    iEvent                           iData

    HMEVENT_SERVER_ATTACH            0
    HMEVENT_SERVER_DETACH            0
    HMEVENT_COMPILATION_OK           0
    HMEVENT_COMPILATION_CANCELLED    0         }

  THMRCIEventCallback = procedure(hInstance: THMInstance; iEvent, iData: longint); cdecl;


  { ListServers:     Lists all running servers (instances of H&M). The list of servers
                     is returned in the callback function: pBuffer is a pointer to a
                     null terminated string which contains the project name and
                     iData is the server ID (THMInstance).

    ServerCount:     Returns the number of running servers.

    GetServerID:     Provides two concurrent methods to get the ID of the desired server.
                     If lpszProjectFile is not empty, the return value is the ID of the
                     server that is working on this project file. If lpszProjectFile is
                     empty, you must provide an index value where 0 <= idx < ServerCount.

    CreateServer:    Creates a new instance of Help & Manual. The parameter lpszProjectfile
                     is optional. It specifies the help project to open. The specified
                     help project must exist.
                     Before you start a new instance, use ListServers or GetServerID to
                     check whether there is an idle instance or an instance that has
                     already opened the help project. CreateServer returns the ID of the
                     created instance of Help & Manual or 0 if the function fails.

    PingServer:      Tests whether the specified instance is still available and
                     idle (e.g. not compiling a help project at the moment). The function
                     returns 1 if the server is available and idle, it returns a number < 0
                     if the server is not available or busy. }

  procedure SetEventCallback(CallbackFunc: THMRCIEventCallback); cdecl external DLLName;
  procedure ListServers(CallbackFunc: THMRCIDataCallback; AppData: longint); cdecl external DLLName;
  function  ServerCount: longint; cdecl external DLLName;
  function  GetServerID(lpszProjectFile: pchar; idx: longint): THMInstance; cdecl external DLLName;
  function  CreateServer(lpszProjectFile: pchar): THMInstance; cdecl external DLLName;
  function  PingServer(hInstance: THMInstance): longint; cdecl external DLLName;

  { The following functions require a server ID (THMInstance) which can be obtained by one
    of the functions above.

    OpenProject:         Advises the server to open the specified project file. If
                         server has already opened a project file and changes were made,
                         the server responds with a dialog box whether to save the changes.
                         The function does not return unless the user closes the dialog
                         box or a timeout occurs. The function returns true if the server
                         was able to open the specified file.

    CreateProject:       Advises the server to create and open the specified project file.
                         If the server has already opened a project file and changes were made,
                         the server responds with a dialog box whether to save the changes.
                         If the specified project file does already exist, it is
                         overwritten. The second parameter for the template file is
                         optional. If specified, it must point to an existing .hm3 file.

    SaveProject:         Saves the current project under the specified name. If no file
                         name is specified, the server saves the project under the
                         original file name.

    CompileProject:      Compiles a help file resp. creates the specified output.
                         lpszOutputFile is the target file name as you specify it in
                         the Make dialog of Help & Manual, when you compile a help
                         file manually.

    CloseProject:        Advises the server to close the currently open project file.
                         DiscardChanges specifies whether changes to the project file
                         are discarded or not. If DiscardChanges is FALSE and changes
                         were made, the server responds with a dialog box whether to
                         save the file or not and the function does not return unless
                         the user closes the dialog box or a timeout occurs.
                         If DiscardChanges is TRUE, any changes to the help project
                         are ignored and the file is closed without saving.
                         Function result is true if the file was successfully
                         closed, either with or without saving.

    CloseServer:         Advises the server to close and terminate. If there is an open
                         help project file with pending changes, the server responds
                         with a dialog box and does not terminate. The function returns
                         immediately in either case and there is no return value.

    GetProjectName:      Retrieves the name of the project file a server is working on.
                         The project name is returned in the callback function: pBuffer is a
                         pointer to a null terminated string which contains the project name,
                         iData is 0. pBuffer may be empty if the server/instance has no open file.

    GetConfigInt:        Returns the value of the specified configuration item as integer value.
    GetConfigStr:        Returns the value of the specified configuration item as string.
    SetConfigInt:        Modifies the value of the specified configuration item. TRUE if successful.
    SetConfigStr:        Modifies the value of the specified configuration item. TRUE if successful.  }

  function  OpenProject(hInstance: THMInstance; lpszProjectFile: pchar): bool; cdecl external DLLName;
  function  CloseProject(hInstance: THMInstance; DiscardChanges: bool): bool; cdecl external DLLName;
  function  CreateProject(hInstance: THMInstance; lpszProjectFile, lpszTemplateFile: pchar): bool; cdecl external DLLName;
  function  SaveProject(hInstance: THMInstance; lpszProjectFile: pchar): bool; cdecl external DLLName;
  function  CompileProject(hInstance: THMInstance; lpszOutputFile, lpszLogFile: pchar;
                           iOutputFormat, iInclude: longint; DeleteTemp: bool): bool; cdecl external DLLName;
  procedure CloseServer(hInstance: THMInstance); cdecl external DLLName;
  procedure GetProjectName(hInstance: THMInstance; CallbackFunc: THMRCIDataCallback; AppData: longint); cdecl external DLLName;

  function  GetConfigStr(hInstance: THMInstance; lpszSection, lpszKey: pchar; CallbackFunc: THMRCIDataCallback; AppData: longint): bool; cdecl external DLLName;
  function  SetConfigStr(hInstance: THMInstance; lpszSection, lpszKey, lpszValue: pchar): bool; cdecl external DLLName;
  function  GetConfigInt(hInstance: THMInstance; lpszSection, lpszKey: pchar): longint; cdecl external DLLName;
  function  SetConfigInt(hInstance: THMInstance; lpszSection, lpszKey: pchar; iValue: longint): bool; cdecl external DLLName;

  { The following functions refer to entries in two lists of Help & Manual: the table of
    contents and the list of invisible topics. Both are organized in a tree view structure.
    An entry in one of these lists and the topic record beneath are actually not the same
    but both are intertwined and cannot be separated.

    To avoid confusion, the following functions use the term "ITEM" to refer to a record
    structure that includes both - information about the tree node and - optionally - information
    about the topic record the tree node relates to. This record structure is defined
    as class THMTOPICRECORD in ECHMRCI3_DEF.INC (see remarks in this file for details).

    GetTopicList:        Returns the full list of topics in the current help project. The
                         values are returned in CallbackFunc. CallbackFunc is called once per
                         topic ID (pBuffer is a pointer to a null-terminated string which
                         contains the topic ID, iData is 0). CallbackFunc must return
                         true in order to obtain the next topic ID. The values are not sorted
                         and it is not determined whether a topic is referenced in the TOC or
                         in the invisible topics list.

    GetContextList:      Similar to GetTopicList but returns a list of help context numbers
                         and their associated topic IDs. Only topic IDs are returned which
                         actually have a help context number. The values are returned in
                         CallbackFunc (pBuffer is pointer to a null-terminated string which
                         contains the topic ID, iData is the help context number).

                         Help context numbers associated with topic anchors include
                         the full topic ID (syntax: "topicID#anchorID"). Values are not sorted.
                         Remember that topics which do not have a help context number are not
                         listed by this function.

    GetContextNumber:    Returns the help context number for the specified topic ID.
                         If you only need the help context for a particular topic, this function
                         is faster and more convenient than GetContext list.

                         Return value is the help context number of the topic resp. the anchor.
                         If the topic has no help context or does not exist, the return value is 0,
                         if the topic specifies more than one help context number, the return value
                         is the first number (to get all numbers, use GetContextList).
                         If the server is not available or busy, the return value is < 0.

    GetItemCount:        Retrieves the number of items in the specified list.
                         iList is either HMTOCLIST or HMINVISIBLELIST. The
                         return value is >= 0 if the function succeeds or a
                         value < 0 if it fails (server busy or not available).

    GetItemID:           Retrieves the ID of a topic node. Return value is the handle
                         of the tree item or 0 if the node does not exist. You can retrieve
                         the item handle either by specifying a topicID or (if the topic ID
                         is empty) by specifying the absolute index of the item in the list.
                         A return < 0 indicates that the server is not available or busy.

    GetItemList:         Retrieves a list of item captions and IDs. The values are returned in
                         CallbackFunc (pBuffer is pointer to a null-terminated string which
                         contains the caption of the entry, iData is the ID of the item (same
                         as you get from GetItemID). The function returns true if successful.

    SelectItem:          Selects a topic or chapter entry.

    InsertItem:          Inserts a new topic or chapter entry. The function returns the
                         handle of the new tree item or a value <= 0 if the function failed.

    UpdateItem:          Updates an existing topic or chapter entry. The function returns the
                         handle of the successfully updated item or a value <= 0 if it failed.

    ReadItem:            Retrieves the content of a topic or chapter entry. The data is returned
                         in CallbackFunc (pBuffer is pointer to a PHMTOPICRECORD structure).
                         The function returns true if successful.

    DeleteItem:          Deletes a topic or chapter entry including its child entries.       }

  function  GetTopicList(hInstance: THMInstance; CallbackFunc: THMRCIDataCallback; AppData: longint): bool; cdecl external DLLName;
  function  GetContextList(hInstance: THMInstance; CallbackFunc: THMRCIDataCallback; AppData: longint): bool; cdecl external DLLName;
  function  GetContextNumber(hInstance: THMInstance; lpszTopicID: pchar): longint; cdecl external DLLName;
  function  GetItemList(hInstance: THMInstance; iList: longint; CallbackFunc: THMRCIDataCallback; AppData: longint): bool; cdecl external DLLName;
  function  GetItemCount(hInstance: THMInstance; iList: longint): longint; cdecl external DLLName;
  function  GetItemID(hInstance: THMInstance; iList: longint; lpszTopicID: pchar; iIndex: longint): longint; cdecl external DLLName;
  function  SelectItem(hInstance: THMInstance; hItem: HWND): bool; cdecl external DLLName;
  function  InsertItem(hInstance: THMInstance; iList: longint; hParentItem: HWND; iInsertMode: longint; pTopic: PHMTOPICRECORD): longint; cdecl external DLLName;
  function  UpdateItem(hInstance: THMInstance; hItem: HWND; pTopic: PHMTOPICRECORD): longint; cdecl external DLLName;
  function  ReadItem(hInstance: THMInstance; hItem: HWND; iMembers: longint; CallbackFunc: THMRCIDataCallback; AppData: longint): bool; cdecl external DLLName;
  function  DeleteItem(hInstance: THMInstance; hItem: HWND): bool; cdecl external DLLName;

implementation

end.
