unit uHelpmanRCI;

interface

uses
  Windows, SysUtils, Classes, Menus, Actnlist, Graphics, 
  {$IFDEF VER130}
  DsgnIntf,                       // Delphi 5
  {$ELSE}
  DesignEditors, DesignIntf,      // Delphi 6 and higher
  {$ENDIF}
  ToolsAPI;

type
  THelpContextProperty = class(TIntegerProperty)
    function  GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  THelpmanRCI = class(TNotifierObject, IOTAWizard)
  private
    fHMFormWizardItem: TMenuItem;
    procedure AddMenuItems;
    procedure RemoveMenuItems;
    procedure ActionExecute(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute;
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;
  end;

  procedure Register;

{$R helpmanrci.res}

implementation

uses
  uHelpmanRCIMain;

const
  HMFormWizardCaption = 'Help && Manual Project Wizard...';

procedure Register;
begin
  RegisterPackageWizard(THelpmanRCI.Create);
  RegisterPropertyEditor(TypeInfo(THelpContext), nil, 'HelpContext', THelpContextProperty);
end;



{ THelpContextProperty --------------------------------------------------}

function THelpContextProperty.GetAttributes: TPropertyAttributes;
begin
  result := inherited GetAttributes + [paDialog];
end;

procedure THelpContextProperty.Edit;
var
  aHelpContext: integer;
begin
  aHelpContext := GetOrdValue;
  if HelpmanRCIExpert(emPropertyEditor, aHelpContext, GetComponent(0)) then SetOrdValue(aHelpContext);
end;


{ THelpmanRCI ---------------------------------------------------------------}

constructor THelpmanRCI.Create;
begin
  inherited;
  AddMenuItems;
end;

destructor THelpmanRCI.Destroy;
begin
  RemoveMenuItems;
  inherited;
end;

procedure THelpmanRCI.AddMenuItems;
var
  ToolsToolsItem: TMenuItem;
  aBitmap: TBitmap;
  bi: integer;

  function FindToolsToolsItem: TMenuItem;
  var
    i, l: integer;
  begin
    result := nil;
    with (BorlandIDEServices as INTAServices).MainMenu do
      for i := 0 to items.count-1 do
      begin
        if SameText(items[i].name, 'ToolsMenu') then
        begin
          for l := 0 to items[i].count-1 do
            if SameText(items[i].items[l].name, 'ToolsToolsItem') then
            begin
                 result := items[i].items[l];
                 break;
            end;
          break;  {i loop}
        end;
      end;
  end;

  function AddIDEAction(const ACaption: string; AOnExecute: TNotifyEvent; ImageIndex: integer): TAction;
  var
    ActionList: TCustomActionList;
  begin
    ActionList := (BorlandIDEServices as INTAServices).ActionList;
    Result := TAction.Create(nil);
    Result.ActionList := ActionList;
    Result.Caption := ACaption;
    Result.OnExecute := AOnExecute;
    Result.ImageIndex := ImageIndex;
  end;

begin
  if assigned(fHMFormWizardItem) then exit;
  ToolsToolsItem := FindToolsToolsItem;
  if Assigned(ToolsToolsItem) then
  begin
    aBitmap := TBitmap.create;
    try
      bi := -1;
      aBitmap.loadfromresourcename(hInstance, 'ICOHELPMANRCI');
      bi := (BorlandIDEServices as INTAServices).ImageList.AddMasked(aBitmap, clNone);
    finally
      aBitmap.free;
    end;

    fHMFormWizardItem := NewItem('', 0, False, True, nil, 0, '');
    fHMFormWizardItem.Action := AddIDEAction(HMFormWizardCaption, ActionExecute, bi);
    { Insert wizard menu item before "Configure Tools" }
    ToolsToolsItem.parent.insert(ToolsToolsItem.Menuindex, fHMFormWizardItem);
  end;
end;

procedure THelpmanRCI.RemoveMenuItems;
begin
  if assigned(fHMFormWizardItem) then FreeAndNil(fHMFormWizardItem);
end;

{ Unique descriptive name identifying this wizard }
function THelpmanRCI.GetName: string;
begin
  Result := 'Help & Manual 3 Remote Control Interface';
end;

{ Wizard state - always enabled }
function THelpmanRCI.GetState: TWizardState;
begin
  Result := [wsEnabled];
end;

{ Wizards unique ID string }
function THelpmanRCI.GetIDString: string;
begin
  Result := 'ECS.HelpmanRCI3';
end;

procedure THelpmanRCI.ActionExecute(Sender: TObject);
begin
  Execute;
end;

{ Start wizard }
procedure THelpmanRCI.Execute;
var
   dummy: integer;
begin
  HelpmanRCIExpert(emWizard, dummy, nil);
end;


end.
