unit uHelpmanRCIMain;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  Menus, StdCtrls, Buttons, Dialogs, ExtCtrls, ComCtrls, ImgList,
  ToolsAPI, TypInfo, {LibIntf, //D5} echmrci3_intf;

type
  THelpmanRCIExpertMode = (emWizard, emPropertyEditor);
  THelpItemState = (isCreateTopic, isConnectTopic, isConnected, isBroken, isIgnore);

  PTreeHelpItem = ^TTreeHelpItem;  { Record structure for treenode data }
  TTreeHelpItem = record
    Component: TComponent;
    HelpContext: THelpContext;
    AssociatedTopic: string;                
    State: THelpItemState;
  end;

  TFrmHelpmanRCIMain = class(TForm)
    pnlBottom: TPanel;
    pnlButtonsAlign: TPanel;
    BtnPrev: TButton;
    btnNext: TButton;
    BtnCancel: TButton;
    btnHelp: TButton;
    ilControlSymbols: TImageList;
    btnConfig: TButton;
    Bevel6: TBevel;
    Shape1: TShape;
    Bevel7: TBevel;
    lbStep: TLabel;
    lbStepShadow: TLabel;
    nbWizard: TNotebook;
    imgStep: TImage;
    ilSteps: TImageList;
    PnlCalcConstraints: TPanel;
    tvFormControls: TTreeView;
    pnlControlInfo: TPanel;
    Label10: TLabel;
    Label13: TLabel;
    pbControlInfo: TPaintBox;
    btnDisplayTopic: TSpeedButton;
    CbControlTopic: TComboBox;
    rbControlStatus: TRadioGroup;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    EdAutoHelpIncrement: TEdit;
    EdAutoHelpStart: TEdit;
    Panel2: TPanel;
    sbReport: TScrollBox;
    pbReport: TPaintBox;
    Label4: TLabel;
    cbForms: TComboBox;
    Label2: TLabel;
    cbHelpProject: TComboBox;
    btnSelectProject: TButton;
    Label1: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    shWarning: TShape;
    shWarningShadow: TShape;
    procedure FormCreate(Sender: TObject);
    function  Application_HelpFile(aProject: IOTAProject): string;
    procedure InitWizard;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ClearTreeData(aTree: TTreeview);
    procedure BtnPrevNextClick(Sender: TObject);
    function  ExitPage(aPage: integer): boolean;
    function  ClassToImageindex(ClassRef: TClass): integer;
    function  LaunchHelpman(const Filename: string): boolean;
    function  CreateContextTopic(aInstance: THMInstance; aTopicID: string; aContext: integer; aPersistent: TPersistent; aParentNode: longint; DoHilight: boolean): longint;
    function  AddFormControlItem(aComponent: TComponent; aTree: TTreeview; aParentNode: TTreeNode; ContextList: TStringlist): TTreeNode;
    function  CreateFormTopics: boolean;
    function  FindMRUMatch(const MatchName: string): string;
    procedure AddMRUFile(const Filename, Association: string);
    procedure InitPage(Sender: TObject; aPage: integer);
    procedure SelectDocuType(Sender: TObject);
    procedure btnSelectProjectClick(Sender: TObject);
    procedure UpdateControlInfo(Node: TTreeNode);
    procedure tvFormControlsChange(Sender: TObject; Node: TTreeNode);
    procedure tvFormControlsAdvancedCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; Stage: TCustomDrawStage;
      var PaintImages, DefaultDraw: Boolean);
    procedure cbComponentsChange(Sender: TObject);
    procedure rbControlStatusClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbHelpProjectDropDown(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
    procedure CbControlTopicChange(Sender: TObject);
    procedure btnDisplayTopicClick(Sender: TObject);
    procedure tvFormControlsChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
    procedure BtnCancelClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnConfigClick(Sender: TObject);
    procedure pbReportPaint(Sender: TObject);
    procedure EdAutoHelpExit(Sender: TObject);
    procedure EdAutoHelpStartChange(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure cbHelpProjectExit(Sender: TObject);
  private
  public
    sWizardHelpfile: string;
    sBaseRegistryKey: string;
    sProjectName, sProjectHelpFile: string;
    PageOrder: string;
    MRUList: TStringlist;
    WizardInstance: THMInstance;
    WizardBusy: boolean;
    PropEditPos: TPoint;

    { Configuration values }
    cfgAppCreateUnnamed, cfgAppCreateNonwindowed,
    cfgPropEditSmartInsert, cfgPropEditBringToFront: boolean;
    cfgAppTopicPrefix: string;
    cfgAppIgnoreClasses: TStringlist;

    procedure SetBusy(value: boolean);
  end;

  function  HelpmanRCIExpert(aMode: THelpmanRCIExpertMode; var aHelpContext: integer; aComponent: TPersistent): boolean;
  function  DrawHTML(r: TRect; aCanvas: TCanvas; const text: string; TabWidth: integer; DoPaint: boolean): integer;
  function  ExpandName(const base, value: string): string;
  procedure EnableControl(Sender: TControl; Value: boolean);
  function  GetHelpContext(aPersistent: TPersistent; var aHelpContext: THelpContext): boolean;
  function  SetHelpContext(aPersistent: TPersistent; const aHelpContext: THelpContext): boolean;

  { Callback function for DLL } 
  function  DataCallback(pBuffer: pointer; iData: integer; AppData: integer): bool; cdecl;

var
  FrmHelpmanRCIMain: TFrmHelpmanRCIMain;

implementation

{$R *.DFM}

uses
  Registry, uHelpmanRCIConfig, uHelpmanRCISingleInsert;

const
  HMRCIREGKEY = 'HelpmanRCI';
  HMRCIHELPFILE = 'HelpmanRCI.hlp';
  WizardCaptions: array [0..3] of string =
                  ('Select Form or Data Module',
                   'Select Controls to create Help for',
                   'Help Context Numbering',
                   'Report (final slide)');


{ HelpmanRCIExpert:
  Starts the wizard. The aMode parameter determines whether we are in

  emWizard mode:
    user started the wizard from the Help menu -> Multiple topics (form) mode

  emPropertyEditor mode:
    user clicked a THelpContext property -> Single topic mode
    the function result defines whether to change the component property or not }

function HelpmanRCIExpert(aMode: THelpmanRCIExpertMode; var aHelpContext: integer; aComponent: TPersistent): boolean;
begin
  result := false;
  try
    FrmHelpmanRCIMain := TFrmHelpmanRCIMain.Create(Application);
    with FrmHelpmanRCIMain do
    try
       case aMode of
       emWizard:
         begin
              ShowModal;
              result := true;
         end;
       emPropertyEditor:
         begin
              with TFrmHelpmanRCISingleInsert.create(FrmHelpmanRCIMain) do
              try
                 if (PropEditPos.x <> -1) or (PropEditPos.y <> -1) then setbounds(PropEditPos.x, PropEditPos.y, Width, Height);
                 result := SingleInsert(aHelpContext, aComponent);
              finally
                 PropEditPos := point(Left, Top);
                 free;
              end;
         end;
       end;
    finally
       Free;
    end;
  except
    on EAbort do { Ignore }
  end;
end;

{ DataCallback:
  General data callback function for various DLL commands }

function DataCallback(pBuffer: pointer; iData: integer; AppData: integer): bool; cdecl;
begin
     if iData = 0 then TStringlist(AppData).Add(pchar(pBuffer))
                  else TStringlist(AppData).AddObject(pchar(pBuffer), pointer(iData));
     result := true;
end;

{ DrawHTML:
  A tiny procedure to paint HTML formatted text to a canvas.
  Used to display control and component information.
  The function returns the required height for a fixed width. }

function DrawHTML(r: TRect; aCanvas: TCanvas; const text: string; TabWidth: integer; DoPaint: boolean): integer;
var
  p: PChar;
  c: Char;
  x, y, wc, hc: Integer;
  aword, code, oldFontName: string;

  procedure PaintWord;
  begin
    if aword <> '' then
    begin
       wc := aCanvas.TextWidth(trimright(aword));
       if x + wc > r.Right then
       begin
            x := r.Left;
            inc(y, hc);
            aword := trimleft(aword);
       end;
       wc := aCanvas.TextWidth(aword);
       if DoPaint and (y + hc < r.Bottom) then aCanvas.TextOut(x, y, aword);
       inc(x, wc);
       { Italic style requires extra space, approximately half the width of
       the last character - this is not really exact but of sufficient accuracy }
       if (fsItalic in aCanvas.Font.Style) then inc(x, aCanvas.TextWidth(aWord[length(aWord)]) div 2);
       aword := '';
    end;
  end;

begin
  p := PChar(text);
  x := r.Left;
  y := r.Top;
  aword := '';
  hc := aCanvas.TextHeight('Ag');
  if p <> nil then
  repeat
      c := p^;
      case c of
      '<': begin
                PaintWord;
                code := '';
                inc(p);
                while (p^ <> '>') and (p^ <> #0) do
                begin
                     code := code + lowercase(p^);
                     inc(p);
                end;
                     if code = 'b'  then aCanvas.Font.Style := aCanvas.Font.Style + [fsBold]
                else if code = 'i'  then aCanvas.Font.Style := aCanvas.Font.Style + [fsItalic]
                else if code = 'u'  then aCanvas.Font.Style := aCanvas.Font.Style + [fsUnderline]
                else if code = '/b' then aCanvas.Font.Style := aCanvas.Font.Style - [fsBold]
                else if code = '/i' then aCanvas.Font.Style := aCanvas.Font.Style - [fsItalic]
                else if code = '/u' then aCanvas.Font.Style := aCanvas.Font.Style - [fsUnderline]
                else if code = 'tab' then
                begin
                     x := r.left + ((((x - r.left) div TabWidth) + 1) * TabWidth);
                end
                else if code = 'li' then
                begin
                     if x > r.left then inc(y, hc);
                     x := r.left;
                     aWord := ' ';
                     oldFontName := aCanvas.Font.Name;
                     aCanvas.Font.Name := 'Symbol';
                     PaintWord;
                     aCanvas.Font.Name := oldFontName;
                end
                else if pos('checked', code) > 0 then
                begin
                     if code = 'checked' then aWord := ''
                     else aWord := 'o';
                     oldFontName := aCanvas.Font.Name;
                     aCanvas.Font.Name := 'WingDings';
                     PaintWord;
                     aCanvas.Font.Name := oldFontName;
                end
                else if code = 'br' then
                begin
                     inc(y, hc);
                     x := r.left;
                end;
           end;
      #10: begin
                PaintWord;
                x := r.Left;
                inc(y, hc);
           end;
      #32,',',';','.','!',':':
           begin
                aword := aword + c;
                PaintWord;
           end;
      #0:  begin
                PaintWord;
           end;
      else begin
                aword := aword + c;
           end;
      end; //case
      inc(p);
   until (c = #0);
   if x > 0 then inc(y, hc);
   result := y;
end;

{ EnableControl:
  Enables or disables a particular control. Edits and combos
  change their color to indicate enabled/disabled style. }

procedure EnableControl(Sender: TControl; Value: boolean);
var
   PropInfo: PPropInfo;
begin
     PropInfo := GetPropInfo(Sender.classinfo, 'Enabled');
     if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkEnumeration)
       then SetOrdProp(Sender, PropInfo, byte(value));

     if (Sender is TCustomEdit) or (Sender is TCustomCombobox) then
     begin
          PropInfo := GetPropInfo(Sender.classinfo, 'Color');
          if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkInteger) then
          begin
               if value then SetOrdProp(Sender, PropInfo, clWindow)
                        else SetOrdProp(Sender, PropInfo, clBtnFace);
          end;
     end;
end;

{ ExpandName: expands a given file name relative
  to BASE and not to the current directory }
function ExpandName(const base, value: string): string;
var
   LastDir: string;
begin
     LastDir := GetCurrentDir;
     SetCurrentDir(ExtractFileDir(base));
     result := ExpandFileName(value);
     SetCurrentDir(LastDir);
end;

{ GetHelpContext + SetHelpContext:
  Read and write the HelpContext property by name }

function GetHelpContext(aPersistent: TPersistent; var aHelpContext: THelpContext): boolean;
var
   PropInfo: PPropInfo;
begin
     try
       PropInfo := GetPropInfo(aPersistent.classinfo, 'HelpContext');
       result := (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkInteger);
       if result then aHelpContext := GetOrdProp(aPersistent, PropInfo);
     except
       result := false;
     end;
end;

function SetHelpContext(aPersistent: TPersistent; const aHelpContext: THelpContext): boolean;
var
   PropInfo: PPropInfo;
begin
     try
       PropInfo := GetPropInfo(aPersistent.classinfo, 'HelpContext');
       result := (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkInteger);
       if result then SetOrdProp(aPersistent, PropInfo, aHelpContext);
     except
       result := false;
     end;
end;

{ GetStringProp:
  Error trapped version of GetStrProp }

function GetStringProp(aPersistent: TPersistent; const aPropName: string): string;
const
   ValidTypes = [tkChar, tkString, tkWChar, tkLString, tkWString];
var
   PropInfo: PPropInfo;
begin
     result := '';
     try
       PropInfo := GetPropInfo(aPersistent.classinfo, aPropName);
       if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in ValidTypes)
         then result := GetStrProp(aPersistent, PropInfo);
     except
     end;
end;

{ FormCreate:
  Read position, size and MRU list. Form contraints are calculated at run
  time - based on the width and height of the NOT aligned pnlCalcConstraints.
  This is neccessary to ensure proper alignment of all controls. Anchors
  and hard coded constraints don't work well if this wizard is compiled with
  a different screen resolution (small vs. large fonts) than it was designed. }

procedure TFrmHelpmanRCIMain.FormCreate(Sender: TObject);
var
   i: integer;
   s: string;
   RegKeys: TStringlist;
   IDERoot, IDEIdentifier: string;
begin
   WizardBusy := false;
   MRUList := TStringlist.create;
   cfgAppIgnoreClasses := TStringlist.create;

   sBaseRegistryKey := (BorlandIDEServices as IOTAServices).GetBaseRegistryKey;

   { Default configuration for the first start }
   cfgAppCreateUnnamed := true;
   cfgAppCreateNonwindowed := true;
   cfgPropEditSmartInsert := false;
   cfgPropEditBringToFront := true; //switch focus to H&M
   cfgAppIgnoreClasses.text := '';
   cfgAppTopicPrefix := '';  // = use control name for topic ID
   PropEditPos := point(-1,-1);

   Constraints.MinWidth := pnlCalcConstraints.width + 16;
   Constraints.MinHeight := nbWizard.top + pnlCalcConstraints.height + pnlBottom.height + 32;
   pnlCalcConstraints.align := alClient;

   { Center dialog and initialize with defaults }
   Setbounds((screen.width-width) div 2, (screen.height-height) div 2, Constraints.MinWidth, Constraints.MinHeight);

   { Read position and size }
   with TRegistry.Create(KEY_READ) do
   try

     { Read root directory of IDE and get the product identifier (DELPHI or CPPBUILDER}
     if (BorlandIDEServices as IOTAServices).GetProductIdentifier = 'Delphi'
       then IDEIdentifier := '$(DELPHI)'
       else IDEIdentifier := '$(BCB)';

     IDERoot := '';
     RootKey := HKEY_LOCAL_MACHINE;
     if OpenKeyReadOnly(sBaseRegistryKey) then
     try
        IDERoot := ReadString('RootDir');
     finally
        CloseKey;
     end;

     RootKey := HKEY_CURRENT_USER;
     if OpenKeyReadonly(sBaseRegistryKey + '\' + HMRCIREGKEY) then
     try
       RegKeys := TStringlist.create;
       try
         GetValueNames(RegKeys);
         for i := 0 to RegKeys.count-1 do if AnsiStrLIComp(pchar(RegKeys[i]), 'MRU_', 4) = 0 then
         begin
            s := ReadString(RegKeys[i]);

          { MRUList is stored in the format <helprojectfile>;ProjectName
            The part after the ";" is used to find the most appropriate
            help file name for the current Delphi project. }

            if (s <> '')
              and fileexists(copy(s,1,pos(';',S)-1))
              and (MRUList.indexof(s) = -1)
              then MRUList.add(s);
         end;
       finally
         RegKeys.free;
       end;

       try
         Top := ReadInteger('Top');
         Left := ReadInteger('Left');
         Width := ReadInteger('Width');
         Height := ReadInteger('Height');
         PropEditPos.x := ReadInteger('PropEditLeft');
         PropEditPos.y := ReadInteger('PropEditTop');

         cfgAppCreateUnnamed := ReadBool('cfgAppCreateUnnamed');
         cfgAppCreateNonwindowed := ReadBool('cfgAppCreateNonwindowed');
         cfgAppIgnoreClasses.text := ReadString('cfgAppIgnoreClasses');
         cfgAppTopicPrefix := ReadString('cfgAppTopicPrefix');
         cfgPropEditSmartInsert := ReadBool('cfgPropEditSmartInsert');
         cfgPropEditBringToFront := ReadBool('cfgPropEditBringToFront');
         sWizardHelpFile := ReadString('HelpFile');
       except
       end;
     finally
       CloseKey;
     end;

     if (sWizardHelpFile = '') or not FileExists(sWizardHelpFile) then
     begin
          { Our best guess is that our help file is installed in the \Help folder.
            If this fails, we check the installed packages for the path of our own
            package. This is our second best guess: the help file may be in the
            same folder as the package. If this fails as well, we give up and use
            only the file name without path (this works as well, but the user will
            have to search for the help file the first time) }

          S := IDERoot + '\Help\' + HMRCIHELPFILE;
          if FileExists(S) then sWizardHelpFile := S
          else if OpenKeyReadOnly(sBaseRegistryKey + '\Known Packages') then
          try
             RegKeys := TStringlist.create;
             try
                GetValueNames(RegKeys);
                for i := 0 to RegKeys.count-1 do if AnsiStrLIComp(pchar(extractFilename(RegKeys[i])), 'HelpmanRCI_', 11) = 0 then
                begin
                     S := ExtractFilePath(StringReplace(Regkeys[i], IDEIdentifier, IDERoot, [rfReplaceAll, rfIgnoreCase])) + HMRCIHELPFILE;
                     if FileExists(S) then sWizardHelpFile := S;
                     break;
                end;
             finally
                RegKeys.free;
             end;
          finally
             CloseKey;
          end;
     end;

   finally
     Free;
   end;

   if sWizardHelpFile <> ''
     then self.Helpfile := sWizardHelpFile //file with path
     else self.Helpfile := HMRCIHELPFILE;  //default without path

   InitWizard;
   nbWizard.PageIndex := 0;  //switch to first page
   InitPage(nil, 0);
end;

procedure TFrmHelpmanRCIMain.FormDestroy(Sender: TObject);
var
   i: integer;
begin
  ClearTreeData(tvFormControls);

  with TRegistry.Create do
  try
    if OpenKey(sBaseRegistryKey + '\' + HMRCIREGKEY, true) then
    begin
      try
         WriteInteger('Top', Top);
         WriteInteger('Left', Left);
         WriteInteger('Width', Width);
         WriteInteger('Height', Height);
         if PropEditPos.x <> -1 then WriteInteger('PropEditLeft', PropEditPos.x);
         if PropEditPos.y <> -1 then WriteInteger('PropEditTop', PropEditPos.y);

         WriteBool('cfgAppCreateUnnamed', cfgAppCreateUnnamed);
         WriteBool('cfgAppCreateNonwindowed', cfgAppCreateNonwindowed);
         WriteString('cfgAppIgnoreClasses', cfgAppIgnoreClasses.text);
         WriteString('cfgAppTopicPrefix', cfgAppTopicPrefix);
         WriteBool('cfgPropEditSmartInsert', cfgPropEditSmartInsert);
         WriteBool('cfgPropEditBringToFront', cfgPropEditBringToFront);

         WriteString('HelpFile', sWizardHelpFile);

         for i := 0 to MRUList.count-1 do if i < 10 then WriteString('MRU_' + inttostr(i), MRUList[i]);
      except
         { ignore }
      end;
      CloseKey;
    end;
  finally
    Free;
  end;

  cfgAppIgnoreClasses.free;
  MRUList.free;
end;


{ Application_HelpFile:
  Attempts to retrieve Application.Helpfile from the project unit.

  Remember that this value is _very_ vague!! Application.Helpfile
  could be set anywhere and anytime at runtime, so we never know
  whether this design time value is really appropriate. Hence
  we only use it as a suggestion for help project selection on
  the first wizard page and do not dare to change it! }

function TFrmHelpmanRCIMain.Application_HelpFile(aProject: IOTAProject): string;
const
   AppHelpFile = 'Application.HelpFile:=';
var
   SourceEditor: IOTASourceEditor;
   Reader: IOTAEditReader;
   aLine: string;
   chr: char;
   IsComment, IsQuoted: boolean;
   i, BytesRead, Position: integer;
begin
     result := '';
     for i := 0 to aProject.GetModuleCount-1 do
     begin
          try
             SourceEditor := aProject.GetModuleFileEditor(i) as IOTASourceEditor;
             if Assigned(SourceEditor) then
             try
                  Reader := SourceEditor.CreateReader;
                  IsComment := false;
                  IsQuoted := false;
                  ALine := '';
                  Position := 0;
                  repeat
                        BytesRead := Reader.GetText(Position, @chr, 1);
                        if BytesRead > 0 then
                        begin
                          case chr of
                          #13,#10: begin
                                        if AnsiStrLIComp(pchar(aLine), AppHelpFile, length(AppHelpFile)) = 0 then
                                        begin
                                             Position := length(AppHelpFile)+2;
                                             while Position <= length(aLine) do
                                               if aLine[Position] = #39 then break
                                               else begin
                                                    result := result + aLine[Position];
                                                    inc(Position);
                                               end;
                                             BytesRead := 0; //break;
                                        end;
                                        aLine := '';
                                   end;
                          '{': IsComment := true;
                          '}': IsComment := false;
                          else begin
                               if (chr = #39) and not IsComment then IsQuoted := not IsQuoted;
                               if not IsComment and ((pos(chr, #9#32) = 0) or IsQuoted) then aLine := aLine + chr;
                          end;
                          end;
                          inc(Position);
                        end;
                  until BytesRead < 1;
             finally
                  Reader := nil;
             end;
          except
          end;
          if result <> '' then break;
     end;
end;

{ InitWizard:
  General initialization procedure - tests whether a project is open at all
  and lists all installed components. }

procedure TFrmHelpmanRCIMain.InitWizard;
var
   ModuleServices: IOTAModuleServices;
   Module: IOTAModule;
   ProjectGroup: IOTAProjectGroup;
   FormEditor: IOTAFormEditor;
   i: Integer;
begin
   ModuleServices := BorlandIDEServices as IOTAModuleServices;

   { Test whether a project is open or not. This is non-critical and only used
     to guess the matching help project or suggest a name for a new project }

   sProjectName := '';
   for i := 0 to ModuleServices.ModuleCount-1 do
   begin
        Module := ModuleServices.Modules[i];
        if Module.QueryInterface(IOTAProjectGroup, ProjectGroup) = S_OK then
        begin
             sProjectName     := ProjectGroup.ActiveProject.FileName;
             sProjectHelpfile := application_helpfile(ProjectGroup.ActiveProject);
             break;
        end;
   end;

   { This one is critical: check which forms are open and whether a FormDesigner is
     available. Available forms are added to cbForms for selection and
     cbForms.items.objects contains a pointer to the FormEditor of the form or data module.

     We rather keep a pointer to IOTAFormEditor than to IOTAFormEditor.GetRootComponent -
     we'll later have to trigger IOTAFormEditor.MarkModified if we change something }

   with cbForms do
   begin
        Items.BeginUpdate;
        for i := 0 to ModuleServices.ModuleCount-1 do
        begin
           try
             FormEditor := ModuleServices.Modules[i].GetModuleFileEditor(1) as IOTAFormEditor;
             if assigned(FormEditor)
               and assigned(FormEditor.GetRootComponent)
               and assigned(TComponent(FormEditor.GetRootComponent.GetComponentHandle))
               then Items.AddObject(changefileext(extractFileName(ModuleServices.Modules[i].FileName), ''),
                                    Pointer(FormEditor));
           except
           end;
        end;
        Items.EndUpdate;
   end;

end;

{ FormClose:
  Saves position, size and MRU list }

procedure TFrmHelpmanRCIMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
end;

{ SetBusy:
  This procedure sets the global variable WizardBusy and enables resp. disables
  the wizard buttons accordingly. WizardBusy is true if we are in a modal loop
  (e.g. parsing form controls or creating help topics) }

procedure TFrmHelpmanRCIMain.SetBusy(value: boolean);
begin
     WizardBusy := value;
     BtnPrev.enabled := not value;    // in a modal loop, we ARE beyond the first page
     BtnNext.enabled := not value;    // BtnNext is always enabled ("Finish" at last)
     BtnHelp.enabled := not value;    // no worry
     BtnConfig.enabled := not value;  // detto
end;


{ ClearTreeData:
  Our treeview nodes all contain pointers to data records. This procedure
  disposes the allocated memory and clears the item list of the treeview. }

procedure TFrmHelpmanRCIMain.ClearTreeData(aTree: TTreeview);
var
   Node: TTreeNode;
begin
     if aTree.items.count > 0 then
     begin
          Node := aTree.items[0];
          while Node <> nil do
          begin
               Dispose(Node.Data);
               Node.Data := nil;
               Node := Node.GetNext;
          end;
          aTree.items.clear;
     end;
end;

{ ClassToImageindex:
  Would be nice if we could use the component palette images for the class. But
  these palette images usually have 24 x 24 pixel which is too large for our
  treeview and looks too ugly if we stretch them to 16 x 16 pixel. Instead
  we use a simple condensed image list to indicate the type of a control.
  ImageIndex 0 means an "unknown" control and displays a question mark for that. }

function TFrmHelpmanRCIMain.ClassToImageindex(ClassRef: TClass): integer;
const
     SimpleClassNames: array [1..20] of string =
      ('tform tcustomform tdatamodule',
       'tcustomgroupbox tcustompanel tnotebook ttabsheet ttoolwindow tpagescroller tscrollbar tscrollingwincontrol',
       'tbuttoncontrol tspeedbutton tcustomupdown',
       'tcustomlabel tcustomstatictext',
       'tcustomedit',
       'tcustomcombobox tcommoncalendar',
       'tcustomlistbox',
       'tcustomcheckbox',
       'tcustomradiogroup tradiobutton',
       'theader theadercontrol tstatusbar', //tcustomcontrolbar
       'tcustomtabcontrol ttabset',
       'tshape tbevel',
       'tdbctrlgrid tcustomgrid',
       'tcustomtreeview tcustomlistview tcustomoutline',
       'tsplitter',
       'tchart tdbchart',
       'timage tanimate tgraphicontrol',
       'ttrackbar tprogressbar',
       'tcustomrichedit',
       'tmenu tmenuitem');
var
   i: integer;
   cName: string;
begin
     result := 0;
     repeat
        cname := lowercase(ClassRef.ClassName);
        for i := 1 to 20 do if pos(cname, SimpleClassNames[i]) > 0 then
        begin
             result := i;
             break;
        end;
        ClassRef := ClassRef.ClassParent;
     until (result > 0) or (cname = 'tcomponent') or (ClassRef = nil);
end;


{ BtnPrevNextClick:
  Whether we go forward or back is defined by the Tag property of Sender.
  The next page to switch to is determined by the string PageOrder. }

procedure TFrmHelpmanRCIMain.BtnPrevNextClick(Sender: TObject);
var
   NextPage: integer;
begin
     try
        NextPage := strtoint( PageOrder[pos(inttostr(nbWizard.PageIndex), PageOrder) + TButton(Sender).tag] );
     except
        NextPage := -1;
     end;

     if (Sender = BtnNext) then
     begin
          if ExitPage(nbWizard.PageIndex) then
          begin
               if NextPage > -1 then
               begin
                    { Change page before init! }
                    nbWizard.PageIndex := NextPage;
                    InitPage(Sender, NextPage);
               end
               else PostMessage(self.handle, WM_CLOSE, 0, 0);  //exit wizard;
          end;
     end
     else begin  { Sender = BtnPrev }
          btnConfig.enabled := pos(inttostr(NextPage), PageOrder) = 1; //only on first page
          btnPrev.enabled   := not btnConfig.enabled;
          btnNext.enabled   := true;
          btnNext.Caption   := 'Next >';
          if NextPage > -1 then nbWizard.PageIndex := NextPage;

          { Re-init first page if user clicked back }
          if NextPage = 0 then InitPage(Sender, NextPage);
     end;
end;


{ ExitPage:
  Tests whether we can exit the current page at all. }

function TFrmHelpmanRCIMain.ExitPage(aPage: integer): boolean;
var
   i: integer;
begin
     result := false;
     case aPage of

     { "TS0" - select documentation type }
     0: begin
          result := cbForms.itemindex > -1;

          { Check for existing file name, prompt if it does not exist and
            if we are to create it, test whether the directory is valid. }
          if result and not FileExists(cbHelpProject.text) then
          begin
               result := MessageDlg('The project file "'+cbHelpProject.text+'" does not exist. Do you want to create it?', mtConfirmation, [mbYes,mbNo], 0) = mrYes;

               { Don't use ForceDirectories - this requires the FileCtrl unit which in turn
                 requires the package VCLX to be included. This function is not important
                 enough to to justify the inclusion of a package.
               if result then
               begin
                    result := ForceDirectories(ExtractFileDir(cbHelpProject.text));
                    if not result then MessageDlg('Failed to create directory!', mtError, [mbOK], 0);
               end;}
          end;

          { Start Help & Manual }
          if result then
          begin
               result := LaunchHelpman(cbHelpProject.text);

               { Add successfully opened files to the MRU list: }
               if result then AddMRUFile(cbHelpProject.text, ExtractFileName(sProjectName));

               { Bring wizard window to front again }
               Application.bringtofront;
          end;

        end;

     { "TS1" - list project forms and select }
     1: begin
          tvFormControlsChanging(tvFormControls, tvFormControls.selected, result);
        end;

     { "TS2" - check auto numbering }
     2: begin
          result := false;
          try
             i := strtoint(EdAutoHelpStart.text);
             if (i > 0) and SetConfigInt(WizardInstance, 'PROJECT', 'AUTOHELPSTART', i) then
             begin
                  i := strtoint(EdAutoHelpIncrement.text);
                  if i > 0 then result := SetConfigInt(WizardInstance, 'PROJECT', 'AUTOHELPINCREMENT', i);
             end;
          except
          end;
          if not result then MessageDlg('Please enter a valid number for the next help context numbers and for subsequent increments.', mtInformation, [mbOK], 0);
        end;

     { "TS3" - Final message }
     3: begin
          result := true;
        end;

     end;  //case
end;

{ LaunchHelpman:
  Starts Help & Manual and opens the specified project.
  If the project does not exist, it is created.
  If the project is already open, it just returns true }

function TFrmHelpmanRCIMain.LaunchHelpman(const Filename: string): boolean;
begin
     WizardInstance := GetServerID(pchar(FileName), 0);
     result := WizardInstance > 0;
     if not result then  //our help file is yet not open, create new instance
     begin
          WizardInstance := CreateServer('');
          if WizardInstance > 0 then
          begin
               if fileexists(Filename)
                 then result := OpenProject(WizardInstance, pchar(Filename))
                 else result := CreateProject(WizardInstance, pchar(Filename), ''); //no template

               if not result then MessageDlg('Cannot open help project ' +Filename, mtError, [mbOK], 0);
          end
          else begin
               MessageDlg('Cannot start Help && Manual.', mtError, [mbOK], 0);
               result := false;
          end;
     end;
end;

{ AddFormControlItem:
  This function is called recursively for each control of a form. It builds
  the treeview items and tests for associated help topics. }

function TFrmHelpmanRCIMain.AddFormControlItem(aComponent: TComponent; aTree: TTreeview; aParentNode: TTreeNode; ContextList: TStringlist): TTreeNode;
var
   MenuItem: TMenuItem;
   DataRec: PTreeHelpItem;
   hc: THelpContext;
   i: integer;
   sClass: string;
   sName: string;

   { Checks whether a menu item is worth a help topic - seperators are not }

   function CheckAddMenuItem(aItem: TMenuItem): boolean;
   begin
        result := (aItem.name <> '') and (aItem.caption <> '-');
   end;

   { Checks whether a control is worth a help topic - unnamed controls are not.
     A TRadioGroup for instance contains unnamed sub-controls of TRadioButton. }

   function CheckAddControl(aControl: TControl): boolean;
   begin
        result := (aControl.name <> '');
   end;

begin
     result := nil;

     { Important interuption: Application.processmessages not only allows the display
       to be freshed, it also processes mouse click events and lets the user click
       the cancel button to abort this loop. If the global variable "WizardBusy" is
       FALSE, the user hit the cancel button and wants to abort. }

     Application.Processmessages;
     if not WizardBusy then exit;

     { 07-29-2002: CheckAddControl is removed from the iteration below and implemented
       here. Even if a TWinControl has NO name (and therefore is not "worth" a help topic,
       it may contain other controls that we have to check. This is the case, for instance,
       with the Raize component "TRzSplitter", which creates two unnamed panels (-> no help
       topic for the panels) which may contain controls we DO want to create help for. }

     if GetHelpContext(aComponent, hc) and
       (((aComponent is TControl) and CheckAddControl(TControl(aComponent))) or
       ((aComponent is TWinControl) and (TWinControl(aComponent).controlcount > 0))) then
     begin
          New(DataRec);
          with DataRec^ do
          begin
               Component := aComponent;
               HelpContext := abs(hc);   //turn minus help context values into plus
               State := isCreateTopic;
               AssociatedTopic := '';

               if HelpContext <> 0 then
               begin
                    i := ContextList.indexofobject(pointer(HelpContext));
                    if i = -1 then State := isBroken
                    else begin
                         AssociatedTopic := ContextList[i];
                         State := isConnected;
                    end;
               end;

               if (not cfgAppCreateUnnamed) and not (Component is TCustomForm) then
               begin
                    sClass := copy(Component.ClassName, 2, length(Component.ClassName)-1);
                    if (AnsiStrLIComp(pchar(Component.Name), pchar(sClass), length(sClass)) = 0) then State := isIgnore;
               end;

               if not cfgAppCreateNonWindowed
                 and not (Component is TWinControl)
                 and not (Component is TMenuItem)
                 and not (Component is TMenu)
                 then State := isIgnore;

               if (cfgAppIgnoreClasses.indexof(Component.ClassName) > -1)
                 then State := isIgnore;

               if (Component.Name = '') and (HelpContext = 0) then State := isIgnore;
          end;

          if aComponent.Name <> '' then sName := aComponent.Name
                                   else sName := aComponent.ClassName + ' (Unnamed)';

          result := aTree.Items.AddChildObject(aParentNode, sName, DataRec);
          if result = nil then Dispose(DataRec)
          else begin
               result.imageindex := ClassToImageindex(aComponent.ClassType);
               result.selectedindex := result.imageindex;
          end;
     end;

     if aComponent is TMenu then
     begin
          if aComponent is TMainMenu then result := aParentNode;
          for i := 0 to TMenu(aComponent).items.count-1 do
          begin
               MenuItem := TMenu(aComponent).items[i];
               if CheckAddMenuItem(MenuItem) then AddFormControlItem(MenuItem, aTree, result, ContextList);
          end;
     end
     else if aComponent is TMenuItem then
     begin
          for i := 0 to TMenuItem(aComponent).count-1 do
          begin
               MenuItem := TMenuItem(aComponent).items[i];
               if CheckAddMenuItem(MenuItem) then AddFormControlItem(MenuItem, aTree, result, ContextList);
          end;
     end
     else begin
          { Menus first - check first level only }
          if (aParentNode = nil) then
            for i := 0 to aComponent.ComponentCount-1 do
              if (aComponent.components[i] is TMenu)
                then AddFormControlItem(aComponent.components[i], aTree, result, ContextList);

          { Other controls }

          if (aComponent is TWinControl) then
            for i := 0 to TWinControl(aComponent).ControlCount-1 do
              AddFormControlItem(TWinControl(aComponent).controls[i], aTree, result, ContextList);
     end;
end;

{ CreateContextTopic:
  Actually inserts the topic in Help & Manual. It returns the ID of the new tree item. }

function TFrmHelpmanRCIMain.CreateContextTopic(aInstance: THMInstance; aTopicID: string; aContext: integer; aPersistent: TPersistent; aParentNode: longint; DoHilight: boolean): longint;
var
   aCaption, aText: string;
   Topic: THMTOPICRECORD;
begin
     { Try to get the object name for the new tree node. If the name is empty, check for
       a caption. If this fails as well, use 'New topic' }
     aCaption := GetStringProp(aPersistent, 'Name');
     if aCaption = '' then aCaption := GetStringProp(aPersistent, 'Caption');
     if aCaption = '' then aCaption := 'New Topic';

     { Try to get the hint if this property exists and use it as topic text }
     aText := GetStringProp(aPersistent, 'Hint');
     if aText = '' then aText := 'Enter topic text here.';

     if aContext = 0 then Topic.Members := HMTOPIC_NEWPOPUP_AUTONUMBERING
     else Topic.Members := HMTOPIC_NEWPOPUP;
     Topic.NodeLevel := 0; //ignored anyway
     Topic.NodeIndex := 0; //ignored anyway
     Topic.NodeImage := 0; //default icon
     Topic.NodeInclude := HMINCLUDE_ALL;
     if DoHilight then Topic.NodeStatus := HMTOPICSTATUS_UNDERCONSTRUCTION
     else Topic.NodeStatus := HMTOPICSTATUS_COMPLETE;
     Topic.NodeCaption := pchar(aCaption);
     Topic.NodeType := HMNODETYPE_TOPIC;
     Topic.NodeData := pchar(aTopicID);
     Topic.Window := 'Popup';
     Topic.HelpContext := aContext;
     Topic.Keywords := '';
     Topic.AKeywords := '';
     Topic.Related := nil;
     Topic.Anchors := nil;
     Topic.ExtraInfo := nil;
     Topic.TextFormat := HMTOPICFORMAT_TEXT;
     Topic.Text := pchar(aText);

     result := InsertItem(aInstance, HMINVISIBLELIST, aParentNode, HMINSERTMODE_ADDCHILD, @Topic);
end;


{ CreateFormTopics:  }

function TFrmHelpmanRCIMain.CreateFormTopics: boolean;
var
   ReportText: string;
   NumCreated, NumConnected, NumWasConnected, NumIgnored: integer;

   Node: TTreeNode;
   FormTopicNode, NewTopicNode: integer;
   S, NewTopicID: string;
   TopicList: TStringlist;
   i: integer;
   IsOverviewTopic: boolean;
begin
     result := false;
     pbReport.Hint := 'Creating topics... please wait!<br>To abort this function, hit <b>Cancel</b>';
     pbReport.refresh;
     Application.ProcessMessages;

     if tvFormControls.items.count <= 1 then
     begin
          ReportText := 'No controls found.';
     end
     else if PingServer(WizardInstance) = 0 then
     begin
          ReportText := 'Help & Manual does not respond.';
     end
     else begin
          ReportText := '<b>Control<tab>Help Context<tab>Topic ID</b><br><br>';
          NumCreated := 0;
          NumConnected := 0;
          NumWasConnected := 0;
          NumIgnored := 0;

          TopicList := TStringlist.create;
          try
            { We need to know the full topic list to create valid new IDs }
            GetTopicList(WizardInstance, DataCallback, integer(TopicList));

            FormTopicNode := 0;
            Node := tvFormControls.items.GetFirstNode;

            while WizardBusy and (Node <> nil) do
            begin
               Application.ProcessMessages;

               { IsOverviewTopic:
                 We attempt to group context sensitive help: the form
                 topic is a top level entry in the invisible list, all
                 control topics are grouped under this form topic. This
                 is simply more convenient to edit. To see an example for
                 open the source of Help & Manual's help file and take a
                 look at the context sensitive help topics - they are
                 organized exactly this way. }

               IsOverviewTopic := Node = tvFormControls.items[0];

               with PTreeHelpItem(Node.Data)^ do
               case State of
               isCreateTopic:
                 begin
                      { If some idiot attempts to create a help topic for an unnamed control: }
                      if Component.Name = '' then NewTopicID := cfgAppTopicPrefix + Component.ClassName
                      else NewTopicID := cfgAppTopicPrefix + Component.Name;

                      if length(NewTopicID) > 30 then NewTopicID := copy(NewTopicID, 1, 30);
                      i := 0;
                      repeat
                        if i = 0 then S := NewTopicID
                        else S := trim(copy(NewTopicID, 1, 30-length(inttostr(i)))) + inttostr(i);
                        inc(i);
                      until TopicList.indexof(S) = -1;

                      { If control has already a help context that is associated with
                        a different topic, remove the current help context number and
                        get a new one. }
                      if AssociatedTopic <> '' then HelpContext := 0;

                      NewTopicNode := CreateContextTopic(WizardInstance, NewTopicID, abs(HelpContext), Component, FormTopicNode, true);
                      if NewTopicNode > 0 then
                      begin
                           if HelpContext = 0 then HelpContext := GetContextNumber(WizardInstance, pchar(NewTopicID));

                           { See remarks above }
                           if IsOverviewTopic then FormTopicNode := NewTopicNode; //used to group the following topics

                           SetHelpContext(Component, HelpContext);
                           result := true;  //sets form modified flag

                           inc(NumCreated);
                           TopicList.add(NewTopicID);
                           ReportText := ReportText + '<checked> ' + Node.Text + '<tab>' + inttostr(HelpContext) + '<tab>' + NewTopicID + '  [New]<br>';
                      end
                      else begin
                           ReportText := ReportText + '<unchecked> ' + Node.Text + '<b> - Failed to create topic ' + NewTopicID + '</b><br>';
                      end;
                 end;
               isConnectTopic,
               isConnected:
                 begin
                      if State = isConnectTopic then
                      begin
                           HelpContext := integer(cbControlTopic.items.objects[ cbControlTopic.items.indexof(AssociatedTopic) ]);

                           SetHelpContext(Component, HelpContext);
                           result := true;  //sets form modified flag

                           inc(NumConnected);
                           ReportText := ReportText + '<checked> ' + Node.Text + '<tab>' + inttostr(HelpContext) + '<tab>' + AssociatedTopic + '  [Connected]<br>';
                      end
                      else begin
                           inc(NumWasConnected);
                      end;

                      { The following needs some additional comment. As described above
                        we attempt to group control topics under a general "form" topic
                        in the invisible topics list. If the form topic does already
                        exist (State = isConnectTopic or isConnected), we first attempt
                        to find this topic in the invisible topics list. "GetItemID" does
                        exactly this. If the form topic is in the TOC and not in the list
                        of invisible topics, we add a SECOND REFERENCE to this _existing_
                        topic - in the invisible topics list. Although "CreateContextTopic"
                        seems to attempt to create a new topic with an existing ID,
                        Help & Manual automatically adds a second topic reference, if the
                        specified topic ID does already exist. We will not get duplicates. }

                      if IsOverviewTopic then
                      begin
                           FormTopicNode := GetItemID(WizardInstance, HMINVISIBLELIST, pchar(AssociatedTopic), 0);
                           if FormTopicNode = 0 then CreateContextTopic(WizardInstance, AssociatedTopic, 0, Component, 0, true);
                      end;
                 end;
               isBroken:
                 begin
                      inc(NumIgnored);
                      ReportText := ReportText + '<unchecked> ' + Node.Text + '<tab><tab>[Broken]<br>';
                 end;
               isIgnore:
                 begin
                      inc(NumIgnored);
                      ReportText := ReportText + '<unchecked> ' + Node.Text + '<tab><tab>[Ignored]<br>';
                 end;
               end;

               Node := Node.GetNext;
            end;

            if not WizardBusy then ReportText := '<b>Topic creation cancelled!</b><br>Please check the topics that have already been created.';

            ReportText := ReportText + '<br><b>Summary:</b><br>' +
                          inttostr(NumCreated) + ' new topics created<br>' +
                          inttostr(NumConnected) + ' controls connected to existing topics<br>' +
                          inttostr(NumWasConnected) + ' controls were already connected to topics<br>' +
                          inttostr(NumIgnored) + ' controls ignored (broken link or set to ignore)';
          finally
            TopicList.free;
          end;  //try after TopicList.create
     end;
     pbReport.hint := ReportText;
     pbReport.refresh;
end;

{ FindMRUMatch:
  Finds a matching project name for a given Delphi project }

function TFrmHelpmanRCIMain.FindMRUMatch(const MatchName: string): string;
var
   i: integer;
begin
     result := '';
     for i := 0 to MRUList.count-1 do
       if AnsiCompareText(copy(MRUList[i], pos(';',MRUList[i])+1, length(MRUList[i])-pos(';',MRUList[i])), MatchName) = 0 then
     begin
          result := copy(MRUList[i], 1, pos(';',MRUList[i])-1);
          break;
     end;
end;

{ AddMRUFile:
  Adds a successfully opened file to the MRU List }

procedure TFrmHelpmanRCIMain.AddMRUFile(const Filename, Association: string);
var
   S: string;
begin
     S := Filename;
     if Association <> '' then S := S + ';' + Association;
     if MRUList.indexof(S) > -1 then MRUList.delete(MRUList.indexof(S));
     MRUList.insert(0, S);
end;

{ InitPage:
  Initializes the current wizard page }

procedure TFrmHelpmanRCIMain.InitPage(Sender: TObject; aPage: integer);
var
   FormEditor: IOTAFormEditor;
   str: TStringlist;
   S: string;
begin
     case aPage of

     { "TS0" - select documentation type }
     0: begin
          cbForms.enabled := cbForms.items.count > 0;
          cbForms.ParentColor := not cbForms.enabled;

          { PageOrder defines which TabSheets are available and in
            which order they appear. We have room for 10 tabsheets. }

          PageOrder := '0';  //no wizard available
          WizardInstance := 0; //reset

          if Sender <> cbHelpProject then
          begin
               S := ''; //suggested help file
               if sProjectName <> '' then
               begin
                    S := FindMRUMatch(ExtractFileName(sProjectName));
                    if (S = '') and (sProjectHelpFile <> '') then S := ChangeFileExt(sProjectHelpFile, '.hm3');
                    if S = '' then S := ChangeFileExt(sProjectName, '.hm3');
               end;
               if cbForms.itemindex > -1 then
               try
                  FormEditor := IOTAFormEditor(pointer(cbForms.items.objects[cbForms.itemindex]));
                  if TCustomForm(FormEditor.GetRootComponent.GetComponentHandle).HelpFile <> ''
                    then S := ChangeFileExt(TCustomForm(FormEditor.GetRootComponent.GetComponentHandle).HelpFile, '.hm3');
               except
               end;

               if S <> '' then cbHelpProject.text := ExpandName(sProjectName, S)
               else if cbHelpProject.items.count > 0 then cbHelpProject.text := cbHelpProject.items[0];
          end;  // Sender <> cbHelpProject

          PageOrder := '0123';
          btnNext.enabled := (cbHelpProject.text <> '') and (cbForms.itemindex > -1);
        end;

     { "TS1" - list project forms and select }
     1: begin
          { OK, here we start: Help & Manual has started, the desired project file is open.
                 First we read existing topics and their help context numbers. Second we parse
                 the controls on the given form and finally, we combine both and display them
                 in the treeview. }

          str := TStringlist.create;
          tvFormControls.items.beginupdate;
          ClearTreeData(tvFormControls);
          try
             SetBusy(true); //start modal loop, disables buttons, sets WizardBusy to true
             GetContextList(WizardInstance, DataCallback, integer(str));
             FormEditor := IOTAFormEditor(pointer(cbForms.items.objects[cbForms.itemindex]));
             AddFormControlItem(TComponent(FormEditor.GetRootComponent.GetComponentHandle), tvFormControls, nil, str);
          finally
             str.free;
             tvFormControls.FullExpand;
             tvFormControls.perform(WM_VSCROLL, MakeLong(SB_TOP,0), 0); //scroll tree to top
             tvFormControls.items.endupdate;
             SetBusy(false); //end modal loop, enables buttons
          end;

          with CbControlTopic do
          begin
             sorted := false;
             items.clear;
             items.beginupdate;
             GetContextList(WizardInstance, DataCallback, integer(items));
             sorted := true;
             items.endupdate;
          end;

          tvFormControlsChange(tvFormControls, nil);
          btnNext.enabled := true; //finish
        end;

     { "TS2" - check auto-numbering }
     2: begin
          EdAutoHelpStart.Text     := inttostr(GetConfigInt(WizardInstance, 'PROJECT', 'AUTOHELPSTART'));
          EdAutoHelpIncrement.Text := inttostr(GetConfigInt(WizardInstance, 'PROJECT', 'AUTOHELPINCREMENT'));
        end;

     { "TS3" - build help topics for form and display result }
     3: begin
          { Let's go! Our control tree is complete, we know the status of every control,
            Help & Manual is already open (if nobody closed it meanwhile) and we are
            ready to create help topics and modify the form controls. This function
            now performs 3 steps:
            1. Disables all buttons except of "Cancel" (we are in a modal loop)
            2. Creates context topics
            3. Enables buttons and displays the result  }

          SetBusy(true);
          try
             FormEditor := IOTAFormEditor(pointer(cbForms.items.objects[cbForms.itemindex]));
             if CreateFormTopics then FormEditor.MarkModified;
          finally
             SetBusy(false);

             { Change page order on the last page: if you the user clicks BACK, the
               wizard returns to the first initial page to start all over again. }
             PageOrder := '03';
          end;
        end;
     end;  //case

     { Enable buttons }
     btnConfig.enabled := pos(inttostr(aPage), PageOrder) = 1; //only on first page
     btnPrev.enabled   := not btnConfig.enabled;

     if (pos(inttostr(aPage), PageOrder) < length(PageOrder)) or (aPage = 0)
       then btnNext.Caption := 'Next >'
       else btnNext.Caption := 'Finish';

     ilSteps.GetBitmap(nbWizard.PageIndex, imgStep.picture.bitmap);
     imgStep.invalidate;
     lbStep.caption := WizardCaptions[nbWizard.Pageindex];
     lbStepShadow.caption := WizardCaptions[nbWizard.Pageindex];
end;

procedure TFrmHelpmanRCIMain.SelectDocuType(Sender: TObject);
begin
     InitPage(Sender, 0);
end;

procedure TFrmHelpmanRCIMain.btnSelectProjectClick(Sender: TObject);
begin
     with TOpenDialog.create(nil) do
     try
        Filter := 'Help & Manual Project Files (*.hm3)|*.hm3';
        FileName := cbHelpProject.text;
        if execute then cbHelpProject.text := FileName;
     finally
        free;
        SelectDocuType(cbHelpProject);
     end;
end;

procedure TFrmHelpmanRCIMain.UpdateControlInfo(Node: TTreeNode);
begin
     if (Node <> nil) then with PTreeHelpItem(Node.Data)^, pbControlInfo do
     begin
          hint := '<b>Name:</b><tab>' + Component.Name + '<br>' +
                  '<b>Class:</b><tab>' + Component.ClassName + '<br>' +
                  '<b>Context:</b><tab>';

          if PTreeHelpItem(Node.Data)^.HelpContext <> 0 then hint := hint + inttostr(PTreeHelpItem(Node.Data)^.HelpContext) + '<br>'
          else hint := hint + '(not assigned)<br>';
          hint := hint + '<br><b>Status:</b><br>';

          case THelpItemState(rbControlStatus.itemindex) of
          isCreateTopic:
            if PTreeHelpItem(Node.Data)^.HelpContext = 0 then hint := hint + 'Create a <u>new topic</u>, get a <u>new help context number</u> and assign it to the control.'
            else if AssociatedTopic = '' then hint := hint + 'Create a <u>new topic</u> and connect control with topic using help context number <b>'+inttostr(PTreeHelpItem(Node.Data)^.HelpContext)+'</b>'
            else hint := hint + 'Create a <u>new topic</u> with <u>new help context number</u>. (Assigned value <b>'+inttostr(PTreeHelpItem(Node.Data)^.HelpContext)+'</b> is associated with topic <b>'+AssociatedTopic+'</b>.)';
          isConnected:    hint := hint + 'This control is up-to-date and mapped to topic "<b>' + cbControlTopic.text + '</b>"';
          isConnectTopic: hint := hint + 'Assign control to topic "<b>' + cbControlTopic.text + '</b>" and update help context property accordingly.';
          isBroken:       hint := hint + '<u>Help link is broken!</u> There is no topic available for context number <b>'+inttostr(PTreeHelpItem(Node.Data)^.HelpContext)+'</b>.';
          isIgnore:       hint := hint + 'Ignore control and do not update.';
          end;

          Invalidate;
     end
     else begin
          pbControlInfo.hint := 'No control selected.<br>Please select a control from the treeview.';
          pbControlInfo.invalidate;
     end;

     EnableControl(cbControlTopic, Node <> nil);
     EnableControl(rbControlStatus, Node <> nil);
     EnableControl(btnDisplayTopic, (Node <> nil) and (CbControlTopic.itemindex > -1));
end;

{ tvFormControlsChange:
  Updates topic display when user selects a control from the treeview.
  Note that cbControlTopic and rbControlStatus store the current values
  temporarily unless another control is selected. The temporary selected
  values are verified and (if ok) stored in tvFormControlsChang_ING_ }

procedure TFrmHelpmanRCIMain.tvFormControlsChange(Sender: TObject; Node: TTreeNode);
begin
     if Node <> nil then with PTreeHelpItem(Node.Data)^, pbControlInfo do
     begin
          rbControlStatus.itemindex := integer(State);
          case State of
          isConnected,
          isConnectTopic: CbControlTopic.itemindex := CbControlTopic.items.indexof(AssociatedTopic);
          else CbControlTopic.itemindex := -1;
          end;
     end
     else begin
          CbControlTopic.itemindex := -1;
          rbControlStatus.itemindex := -1;
     end;
     UpdateControlInfo(Node);
end;

{ tvFormControlsChanging:
  Verifies the selected values and stores them in the data
  record of the currently selected tree item (this is
  tvFormControl.selected, not Node!).
  If the user selected a combination that does not make sense
  (e.g. state "ConnectTopic" without selecting a topic to connect to)
  the change is rejected and the old values are restored. }

procedure TFrmHelpmanRCIMain.tvFormControlsChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
begin
     AllowChange := true;

     if tvFormControls.selected <> nil then
       with PTreeHelpItem(tvFormControls.selected.data)^ do
     begin
          case THelpItemState(rbControlStatus.itemindex) of
          isCreateTopic,
          isBroken,
          isIgnore:
            begin
                 State := THelpItemState(rbControlStatus.itemindex); //non critical anyway
                 {AssociatedTopic :=   Don't update! In case user
                                       switched unintentionally
                                       from "Connected" to "Create",
                                       we can restore the original
                                       value if he corrects the state }
            end;
          isConnectTopic,
          isConnected:
            begin
                 if (cbControlTopic.itemindex > -1) then
                 begin
                      if (AssociatedTopic = cbControlTopic.text)
                        and (HelpContext = integer(cbControlTopic.items.objects[cbControlTopic.itemindex]))
                          then State := isConnected
                          else State := isConnectTopic;
                      AssociatedTopic := cbControlTopic.text;
                 end
                 else begin
                      MessageDlg('Please select a topic to connect this control to!', mtError, [mbOK], 0);
                      AllowChange := false;
                 end;
            end;
          end;
     end;
end;

{ tvFormControlsAdvancedCustomDrawItem:
  Custom draw function for treeview items: use font.color to indicate
  the state of the item. }

procedure TFrmHelpmanRCIMain.tvFormControlsAdvancedCustomDrawItem(
  Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
  Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
var
   r: TRect;
   oldColor, aColor: TColor;
begin
     DefaultDraw := true;
     PaintImages := true;
     if (Node <> nil) and (Stage = cdPostPaint) then
     try
          aColor := Sender.Canvas.Font.color;
          oldColor := aColor;

          case PTreeHelpItem(Node.Data)^.State of
          isCreateTopic:  aColor := clBlue;
          isConnectTopic: aColor := clGreen;
          isBroken:       aColor := clRed;
          isIgnore:       aColor := clSilver;
          end;

          r := Node.DisplayRect(true); //text only

          with Sender.canvas do
          begin
               if not (cdsSelected in State) then Sender.Canvas.Font.color := aColor;
               DrawText(Sender.Canvas.Handle, PChar(Node.Text), Length(Node.Text), R, DT_SINGLELINE or DT_EXTERNALLEADING or DT_CENTER or DT_VCENTER or DT_NOPREFIX);
               Sender.Canvas.Font.color := oldColor;
               DefaultDraw := true;
               PaintImages := true;
          end;
     except
     end;

end;

procedure TFrmHelpmanRCIMain.cbComponentsChange(Sender: TObject);
{var
   bmp: tbitmap;}
begin
{    Unused test function - paints the palette bitmap of the selected component
     bmp := TBitmap.create;
     bmp.width := 24;
     bmp.height := 24;

     LibIntf.DelphiIDE.GetPaletteItembyName(cbComponents.text).Paint(bmp.canvas, -2, -2);
     image1.picture.bitmap.assign(bmp);
     bmp.free; }
end;

procedure TFrmHelpmanRCIMain.CbControlTopicChange(Sender: TObject);
begin
     { On change event is triggered trough treeviewchange as well
       we update only if the user changed the topic manually }
     if ActiveControl = Sender then
     begin
          rbControlStatus.itemindex := integer(isConnectTopic);
          UpdateControlInfo(tvFormControls.selected);
     end;
end;

procedure TFrmHelpmanRCIMain.rbControlStatusClick(Sender: TObject);
begin
     { On change event is triggered trough treeviewchange as well
       we update only if the user changed the status manually }
     try
        if (ActiveControl is TRadioGroup) or (ActiveControl.parent is TRadioGroup) then
        begin
             if tvFormControls.selected <> nil then
                with PTreeHelpItem(tvFormControls.selected.data)^ do
                  if (Component.Name = '') and (THelpItemState(rbControlStatus.itemindex) <> isIgnore) then
                  begin
                       Messagebeep(0);
                       MessageDlg('You cannot create a help topic for an unnamed control!', mtError, [mbOK], 0);
                       rbControlStatus.itemindex := integer(isIgnore);
                  end;
             UpdateControlInfo(tvFormControls.selected);
        end;
     except
        //no error here
     end;
end;

procedure TFrmHelpmanRCIMain.cbHelpProjectDropDown(Sender: TObject);
var
   i, j, l: integer;
   S: string;
begin
     l := 0;
     for i := 0 to MRUList.count-1 do
     begin
          S := copy(MRUList[i], 1, pos(';',MRUList[i])-1);
          if TCombobox(Sender).items.indexof(S) = -1 then
          begin
               TCombobox(Sender).items.add(S);
               j := canvas.textwidth(S) + 8;
               if (j > l) and (j > TCombobox(Sender).width) then l := j;
          end;
     end;
     TCombobox(Sender).perform(CB_SETDROPPEDWIDTH, l, 0);
end;

procedure TFrmHelpmanRCIMain.PaintBoxPaint(Sender: TObject);
begin
     with TPaintBox(Sender) do
     begin
          Canvas.brush.style := bsSolid;
          Canvas.brush.color := clWindow;
          Canvas.FillRect(ClientRect);
          DrawHTML(ClientRect, Canvas, Hint, 60, true);
     end;
end;

procedure TFrmHelpmanRCIMain.btnDisplayTopicClick(Sender: TObject);
var
   TopicID: string;
   Item: HWND;
begin
     TopicID := CbControlTopic.text;
     if pos('#', TopicID) > 0 then TopicID := copy(TopicID, 1, pos('#', TopicID)-1);

     Item := GetItemID(WizardInstance, HMINVISIBLELIST, pchar(TopicID), 0);
     if Item = 0 then Item := GetItemID(WizardInstance, HMTOCLIST, pchar(TopicID), 0);
     if (Item = 0) or not SelectItem(WizardInstance, Item) then
        if PingServer(WizardInstance) = 1
          then MessageDlg('Cannot locate topic "' + TopicID + '". Could have been changed in the meantime.', mtError, [mbOK], 0)
          else MessageDlg('Help & Manual is not responding!', mtError, [mbOK], 0);
end;

{ BtnCancelClick:
  The cancel button has two functions: (a) it aborts a modal loop (building the
  component list or creating help topics) and (b) closes the wizard. The response
  is determined by the global variable "WizardBusy" which is false by default and
  set to true when the wizard starts a modal loop. }

procedure TFrmHelpmanRCIMain.BtnCancelClick(Sender: TObject);
begin
     if WizardBusy then
     begin
          if MessageDlg('Do you want to cancel the current action?', mtConfirmation, [mbYes,mbNo], 0) = mrYes
            then WizardBusy := false; // Don't use SetBusy(false) here!
     end
     else PostMessage(self.handle, WM_CLOSE, 0, 0);
end;

procedure TFrmHelpmanRCIMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
     CanClose := not WizardBusy;
end;

procedure TFrmHelpmanRCIMain.btnConfigClick(Sender: TObject);
begin
     FrmHelpmanRCIConfig := TFrmHelpmanRCIConfig.create(self);
     with FrmHelpmanRCIConfig do
     try
        cbAppCreateUnnamed.checked := cfgAppCreateUnnamed;
        cbAppCreateNonwindowed.checked := cfgAppCreateNonwindowed;
        cbAppIgnoreClasses.checked := cfgAppIgnoreClasses.count > 0;
        if cbAppIgnoreClasses.checked then MeAppIgnoreClasses.lines.assign(cfgAppIgnoreClasses);
        EdAppTopicPrefix.text := cfgAppTopicPrefix;
        cbPropEditSmartInsert.checked := cfgPropEditSmartInsert;
        cbPropEditBringToFront.checked := cfgPropEditBringToFront;

        if ShowModal = mrOK then
        begin
             cfgAppCreateUnnamed := cbAppCreateUnnamed.checked;
             cfgAppCreateNonwindowed := cbAppCreateNonwindowed.checked;
             cfgAppIgnoreClasses.assign(MeAppIgnoreClasses.lines);
             cfgAppTopicPrefix := trim(EdAppTopicPrefix.text);
             cfgPropEditSmartInsert := cbPropEditSmartInsert.checked;
             cfgPropEditBringToFront := cbPropEditBringToFront.checked;
        end;
     finally
        free;
     end;
end;

procedure TFrmHelpmanRCIMain.pbReportPaint(Sender: TObject);
var
   h: integer;
begin
     with pbReport do if tag = 0 then
     begin
          Canvas.brush.style := bsSolid;
          Canvas.brush.color := clWindow;
          Canvas.FillRect(ClientRect);
          h := DrawHTML(ClientRect, Canvas, Hint, pbReport.width div 3, true);
          if h < sbReport.clientheight then h := sbReport.clientheight;
          if h <> height then
          begin
               tag := 1; //prevent repaint
               try
                  height := h;  //triggers invalidate control
                  sbReport.VertScrollBar.range := h;
               finally
                  tag := 0;
               end;
          end;
     end;
end;

procedure TFrmHelpmanRCIMain.EdAutoHelpExit(Sender: TObject);
var
   i: integer;
begin
     try
        i := strtoint(TEdit(Sender).text);
        if i < 0 then i := 0;
        TEdit(Sender).text := inttostr(i);
     except
        MessageDlg('"' + TEdit(Sender).text + '" is not a valid integer number!', mtError, [mbOK], 0);
        TEdit(Sender).text := '0';
     end;
end;

procedure TFrmHelpmanRCIMain.EdAutoHelpStartChange(Sender: TObject);
begin
     btnNext.enabled := (EdAutoHelpStart.text <> '0') and (EdAutoHelpIncrement.text <> '0');
end;

procedure TFrmHelpmanRCIMain.btnHelpClick(Sender: TObject);
begin
     Application.HelpCommand(15,0);
end;

procedure TFrmHelpmanRCIMain.cbHelpProjectExit(Sender: TObject);
begin
     TComboBox(Sender).text := changeFileExt(TComboBox(Sender).text, '.hm3');
end;

end.
