unit uHelpmanRCISingleInsert;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TypInfo, ECHMRCI3_Intf, ExtCtrls;

type
  TFrmHelpmanRCISingleInsert = class(TForm)
    Label2: TLabel;
    cbHelpProject: TComboBox;
    btnSelectProject: TButton;
    btnOK: TButton;
    BtnCancel: TButton;
    btnHelp: TButton;
    GroupBox1: TGroupBox;
    rbCreateTopic: TRadioButton;
    rbConnectTopic: TRadioButton;
    cbContextList: TComboBox;
    lbHelpContext: TLabel;
    btnConfig: TButton;
    Shape1: TShape;
    Bevel7: TBevel;
    lbStep: TLabel;
    lbStepShadow: TLabel;

    function  SingleInsert(var aHelpContext: integer; aComponent: TPersistent): boolean;
    function  InsertTopic(aInstance: THMInstance; var aHelpContext: integer; aComponent: TPersistent): boolean;
    function  ConnectTopic(aInstance: THMInstance; aTopicID: string; doJump: boolean): longint;
    function  RefreshContextList(const aProjectFile: string): THMInstance;

    procedure btnSelectProjectClick(Sender: TObject);
    procedure cbHelpProjectDropDown(Sender: TObject);
    procedure cbHelpProjectExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure rbCreateTopicClick(Sender: TObject);
    procedure cbHelpProjectChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
  private
    { Private declarations }
  public
    SuggestedContext: integer;
    SelectedComponent: TPersistent;
  end;

var
  FrmHelpmanRCISingleInsert: TFrmHelpmanRCISingleInsert;

implementation
uses uHelpmanRCIMain;

{$R *.DFM}

{ SingleInsert:  This is the entry and exit function of this dialog
                 and determines whether to display the dialog box at all
                 or to use SmartInsert (topic creation without dialog).

  This property editor is going to become more complicated than I thought...
  We have to perform several checks here in order to enable SmartInsert.
  Some thoughts and conclusions first:

  1) When this function is executed, Help & Manual may already be running
     because it was started by a previous topic insertion a minute ago.
     In this case the user probably wants to insert just another topic in
     the same help file -> prerequisites for SmartInsert -> don't bother with
     a dialog.

  2) If the conditions for SmartInsert are OK, we still have a problem though:
     the selected control may have a help context or not. In case it's 0 the
     conclusion is simple: we insert a new topic and assign the help context
     number to the control (function return is true = Modified) and "aHelpContext"
     is filled with the new help context number.

  3) However, the control's HelpContext property may not be 0. In this case we
     check whether a topic with this context number exists.

     If Yes: We just switch over to Help & Manual to edit the topic.
     If No:  We create a new topic but use the given help context number.

     In both cases, the function will return True (if the topic can be
     inserted/selected) because of (4).

  4) What if the user has selected multiple controls? Does this qualify for
     SmartInsert? I'd say: yes, why not? The user may want to insert the same
     help topic for two or more controls. In "aHelpContext" we get the help context
     number of the first control in the selection list. If all other controls have the
     same number (e.g. 0) we won't face a problem at all. If not, the user probably
     intends to assign the same context number to all selected controls. So we
     proceed like in step (2) regardless of one or more selected controls.

     The function will return true if we were able to insert OR select the topic,
     which leads to a synchronization of the help context numbers of all selected
     controls.

     If our assumption is wrong, we can still blame the user for this ... :-)

  5) Wait a minute! What if Help & Manual is not even open so that we do not know
     whether a particular topic exists (in case the help context is > 0)?

     We display the dialog in this case. The dialog lets the user select the desired
     help file (with a pretty good suggestion pre-selected). The default selection
     is "Insert topic", regardless of the help context number. If he attempts to
     select "Connect topic", the changed radio button (see OnClick event) causes
     Help & Manual to start, load the context list and so on. }

function TFrmHelpmanRCISingleInsert.SingleInsert(var aHelpContext: integer; aComponent: TPersistent): boolean;
var
   S: string;
   aInst: THMInstance;
begin
     result := false;
     SuggestedContext := abs(aHelpContext);
     SelectedComponent := aComponent;

     if SuggestedContext <> 0
       then lbHelpContext.Caption := '(Use HelpContext #' + inttostr(SuggestedContext) + ')'
       else lbHelpContext.Caption := '(Get new help context number)';

     S := '';
     with FrmHelpmanRCIMain do if sProjectName <> '' then
     begin
          S := FindMRUMatch(ExtractFileName(sProjectName));

          if (S = '') and (sProjectHelpFile <> '') then S := ChangeFileExt(sProjectHelpFile, '.hm3');
          if (S = '') then S := ChangeFileExt(sProjectName, '.hm3');
     end;
     if (S <> '') then cbHelpProject.text := S
     else if cbHelpProject.items.count > 0 then cbHelpProject.text := cbHelpProject.items[0];


     { S is our suggested help project. If this file exists AND is open in Help & Manual,
       we have two possible situations:

       (a) If aHelpContext <> 0 we check whether a topic with this context number exists
           and directly jump to that topic -> no dialog is displayed at all.
       (b) If aHelpContext = 0 we directly insert a new topic (only if SmartInsert is allowed)

       *RefreshContextList* below checks for a running instance of H&M and loads help context
       numbers into cbContextList.items. It returns the ID of the instance of H&M. This
       function actually serves 2 purposes:

       (a) It checks for a running instance -> required condition for SmartInsert
       (b) Loads the context list -> required to determine whether to connect to or insert a topic }

     if S <> '' then aInst := RefreshContextList(S) else aInst := 0;

     if aInst > 0 then
     begin
          if (SuggestedContext <> 0)
            and (cbContextList.items.indexofObject(pointer(SuggestedContext)) > -1) then
          begin
               rbConnectTopic.checked := true;
               with cbContextList do itemindex := items.indexofObject(pointer(SuggestedContext));
               if FrmHelpmanRCIMain.LaunchHelpman(cbHelpProject.text)
                 and (ConnectTopic(aInst, cbContextList.text, true) > 0)
                 then result := true   // -> Result TRUE synchronizes help context numbers of multiple topics
                 else MessageDlg('Cannot select topic ' + S, mtError, [mbOK], 0);
          end
          else begin
               if FrmHelpmanRCIMain.cfgPropEditSmartInsert then
               begin
                    if FrmHelpmanRCIMain.LaunchHelpman(cbHelpProject.text)
                      and InsertTopic(aInst, SuggestedContext, SelectedComponent)
                      then result := true
                      else MessageDlg('Cannot insert new topic!', mtError, [mbOK], 0);
               end
               else result := ShowModal = mrOK;
          end;
     end
     else result := ShowModal = mrOK;

     if result then
     begin
          aHelpContext := SuggestedContext;
          FrmHelpmanRCIMain.AddMRUFile(cbHelpProject.text, ExtractFileName(FrmHelpmanRCIMain.sProjectName));
     end;
end;

function TFrmHelpmanRCISingleInsert.InsertTopic(aInstance: THMInstance; var aHelpContext: integer; aComponent: TPersistent): boolean;
var
   TopicList: TStringlist;
   ControlParent: TPersistent;
   FormTopicNode, NewTopicNode: integer;
   FormTopicID, NewTopicID: string;
   h: THelpContext;
   i: integer;
   s: string;
begin
   TopicList := TStringlist.create;
   try
      RefreshContextList(cbHelpProject.text);
      GetTopicList(aInstance, DataCallback, integer(TopicList));

      { Check whether the project file has enabled automatic help context
        numbering. If not, we need to invent a help context number for this control.
        Take the highest help context number we can find and increment it by 10. }

      if (aHelpContext = 0) and
        ((GetConfigInt(aInstance, 'PROJECT', 'AUTOHELPSTART') = 0)
          or (GetConfigInt(aInstance, 'PROJECT', 'AUTOHELPINCREMENT') = 0)) then
      begin
          for i := 0 to cbContextList.items.count-1 do
            if integer(cbContextList.items.objects[i]) > aHelpContext
              then aHelpContext := integer(cbContextList.items.objects[i]);
          inc(aHelpContext, 10);
      end;

      ControlParent := aComponent;
      while (ControlParent <> nil)
        and (ControlParent is TControl)
        and not (ControlParent is TCustomForm)
        and not (ControlParent is TDataModule) do ControlParent := TControl(ControlParent).Parent;

      { If the control's parent is a form or datamodule and there is already a topic for
        that parent, we will group the new topic below the parent topic in H&M.

        Please note, that - in opposite to the form wizard (main form) - we do
        NOT attempt to create a help topic for the parent form! We just use it.
        What we do, of course, is if the form topic is in the table of contents,
        we add a second reference to this topic in the Invisible Topics List.
        This does not do any harm and helps us to organize context topics better. }

      FormTopicNode := 0;
      if (ControlParent <> nil)      //our control has a form or datamodule as parent
        and (ControlParent <> aComponent)      //the selected control is not the form itself
        and GetHelpContext(ControlParent, h)   //parent has helpcontext property
        and (h <> 0)                                              //and property is <> 0
        and (cbContextList.items.indexofObject(pointer(abs(h))) > -1) then  // and topic exists
      begin
           FormTopicID := cbContextList.items[ cbContextList.items.indexofObject(pointer(abs(h))) ];
           FormTopicNode := GetItemID(aInstance, HMINVISIBLELIST, pchar(FormTopicID), 0);

           if FormTopicNode = 0 then
           begin
                FormTopicNode := FrmHelpmanRCIMain.CreateContextTopic(aInstance,
                                                                      FormTopicID, //creates a second reference
                                                                      abs(h),
                                                                      ControlParent,
                                                                      0, //no parent node
                                                                      TopicList.indexof(FormTopicID) = -1); //only highlight if new
           end;
      end;

      NewTopicID := FrmHelpmanRCIMain.cfgAppTopicPrefix + GetStrProp(aComponent, 'Name');
      if NewTopicID = FrmHelpmanRCIMain.cfgAppTopicPrefix then NewTopicID := FrmHelpmanRCIMain.cfgAppTopicPrefix + 'NewTopic';

      if length(NewTopicID) > 30 then NewTopicID := copy(NewTopicID, 1, 30);
      i := 0;
      repeat
            if i = 0 then S := NewTopicID
            else S := trim(copy(NewTopicID, 1, 30-length(inttostr(i)))) + inttostr(i);
            inc(i);
      until TopicList.indexof(S) = -1;
      NewTopicID := S;

      NewTopicNode := FrmHelpmanRCIMain.CreateContextTopic(aInstance,
                                                           NewTopicID,
                                                           abs(aHelpContext),
                                                           aComponent,
                                                           FormTopicNode, //use form topic as parent
                                                           false);  //do not highlight single topics

      result := NewTopicNode > 0;
      if result then
      begin
           if aHelpContext = 0 then aHelpContext := GetContextNumber(aInstance, pchar(NewTopicID));
           if FrmHelpmanRCIMain.cfgPropEditBringtoFront then SelectItem(aInstance, NewTopicNode);
      end
      else Application.BringToFront;  //just if anything failed

   finally
      TopicList.free;
   end;
end;

function TFrmHelpmanRCISingleInsert.ConnectTopic(aInstance: THMInstance; aTopicID: string; doJump: boolean): longint;
begin
     result := GetItemID(aInstance, HMINVISIBLELIST, pchar(aTopicID), 0);
     if result = 0 then result := GetItemID(aInstance, HMTOCLIST, pchar(aTopicID), 0);
     if (result > 0) and doJump then SelectItem(aInstance, result);
end;

function TFrmHelpmanRCISingleInsert.RefreshContextList(const aProjectFile: string): THMInstance;
begin
     result := GetServerID(pchar(aProjectFile), 0);
     with cbContextList do if result > 0 then
     begin
          items.beginupdate;
          items.clear;
          sorted := false;
          GetContextList(result, Datacallback, integer(items));
          sorted := true;
          items.endupdate;
     end
     else begin
          items.clear;
     end;
end;

procedure TFrmHelpmanRCISingleInsert.btnSelectProjectClick(Sender: TObject);
begin
     with TOpenDialog.create(nil) do
     try
        Filter := 'Help & Manual Project Files (*.hm3)|*.hm3';
        FileName := cbHelpProject.text;
        if execute then cbHelpProject.text := FileName;
     finally
        free;
     end;
end;

procedure TFrmHelpmanRCISingleInsert.cbHelpProjectDropDown(Sender: TObject);
var
   i, j, l: integer;
   S: string;
begin
     l := 0;
     for i := 0 to FrmHelpmanRCIMain.MRUList.count-1 do
     begin
          S := copy(FrmHelpmanRCIMain.MRUList[i], 1, pos(';',FrmHelpmanRCIMain.MRUList[i])-1);
          if TCombobox(Sender).items.indexof(S) = -1 then
          begin
               TCombobox(Sender).items.add(S);
               j := canvas.textwidth(S) + 8;
               if (j > l) and (j > TCombobox(Sender).width) then l := j;
          end;
     end;
     TCombobox(Sender).perform(CB_SETDROPPEDWIDTH, l, 0);
end;

procedure TFrmHelpmanRCISingleInsert.cbHelpProjectExit(Sender: TObject);
begin
     TComboBox(Sender).text := changeFileExt(TComboBox(Sender).text, '.hm3');
end;

procedure TFrmHelpmanRCISingleInsert.FormCreate(Sender: TObject);
begin
     self.HelpFile := FrmHelpmanRCIMain.HelpFile;
     btnConfig.onclick := FrmHelpmanRCIMain.btnConfig.onclick;
     btnHelp.onclick   := FrmHelpmanRCIMain.btnHelp.onclick;
end;

procedure TFrmHelpmanRCISingleInsert.rbCreateTopicClick(Sender: TObject);
var
   SelectionListOK: boolean;
begin
     { This is somewhat critical: if the user selects "Connect topic" we must
       provide a selection list. To do this, we need to launch Help & Manual
       first (in case it is not already open) }

     if rbConnectTopic.checked then
     begin
          { Help project must exist in this case! We cannot select topics from an empty file. }

          SelectionListOK := (cbHelpProject.text <> '') and FileExists(cbHelpProject.text);
          if not SelectionListOK then MessageDlg('Please select an existing help project!', mtError, [mbOK], 0)
          else begin
               SelectionListOK := FrmHelpmanRCIMain.LaunchHelpman(cbHelpProject.text);
               if SelectionListOK then SelectionListOK := RefreshContextList(cbHelpProject.text) > 0;
          end;

          { If it happens that we cannot select topics, either because of an invalid file name
            or a failure to start H&M or a failure to open the file or a failure to retrieve
            the context list, we switch back to "Create". This causes a subsequent execution
            of this OnClick event which, however, doesn't really bother us as it will not
            execute this code again. }

          if NOT SelectionListOK then
          begin
               rbCreateTopic.checked := true;   //-> nested execution of OnClick
               EXIT;
          end;
     end;
     EnableControl(cbContextList, rbConnectTopic.checked);

     { Bring the dialog back into front }
     Application.BringToFront;
end;

procedure TFrmHelpmanRCISingleInsert.cbHelpProjectChange(Sender: TObject);
begin
     RefreshContextList(cbHelpProject.text);
end;

procedure TFrmHelpmanRCISingleInsert.btnOKClick(Sender: TObject);
var
   TopicID: string;
begin
     {ModalResult := mrNone;   OK button does not close the dialog unless we are successful }

     if (cbHelpProject.text = '') then MessageDlg('Please specify a help project name!', mtError, [mbOK], 0);
     begin
          if FrmHelpmanRCIMain.LaunchHelpman(cbHelpProject.Text) then //else -> message in LaunchHelpman
          begin
               if rbConnectTopic.checked then  //connect to existing topic
               begin
                    if (cbContextList.text <> '') then
                    begin
                         if pos('#', cbContextList.text) > 0
                           then TopicID := copy(cbContextList.text, 1, pos('#',cbContextList.text)-1)
                           else TopicID := cbContextList.text;

                         if ConnectTopic(FrmHelpmanRCIMain.WizardInstance, TopicID, true) > 0 then
                         begin
                              { SuggestedContext is a form-wide variable, containing the original
                                helpcontext of the selected control(s) and also used to submit a
                                changed value which is assigned to the selected control(s) by the
                                property editor. Whether the property editor modifies the controls
                                or not is determined by the dialog result. mrOK = modified! }

                              SuggestedContext := integer(cbContextList.items.objects[cbContextList.itemindex]);
                              ModalResult := mrOK;
                         end
                         else MessageDlg('Cannot select topic! Help && Manual is probably busy.', mtError, [mbOK], 0);
                    end
                    else MessageDlg('Please select a topic to connect to!', mtError, [mbOK], 0);
               end
               else begin  //insert a new topic
                    if InsertTopic(FrmHelpmanRCIMain.WizardInstance, SuggestedContext, SelectedComponent) then
                    begin
                         { SuggestedContext has already been filled in InsertTopic, we
                           just need to return a positive modal result set the changes. }

                         ModalResult := mrOK;
                    end
                    else MessageDlg('Cannot insert new topic!', mtError, [mbOK], 0);
               end;  //end rbInsertTopic.checked
          end;  //end LaunchHelpman
     end;

     { Bring the dialog back into view if anything failed }
     if ModalResult <> mrOK then Application.BringToFront;
end;

end.
